@@ -1,5 +1,5 @@
 # This file is part of Pebble.
-# Copyright (c) 2013-2023, Matteo Cafasso
+# Copyright (c) 2013-2024, Matteo Cafasso
 
 # Pebble is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License
@@ -14,15 +14,24 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with Pebble.  If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Callable
 from functools import wraps
-from traceback import format_exc
 from concurrent.futures import Future
+from typing import Callable, Optional, overload
 
-from pebble.common import launch_thread, SUCCESS
+from pebble import common
+from pebble.pool.thread import ThreadPool
 
-
-def thread(*args, **kwargs) -> Callable:
+@overload
+def thread(func: common.CallableType) -> common.ThreadDecoratorReturnType:
+    ...
+@overload
+def thread(
+        name: Optional[str] = None,
+        daemon: bool = True,
+        pool: Optional[ThreadPool] = None
+) -> common.ThreadDecoratorParamsReturnType:
+    ...
+def thread(*args, **kwargs):
     """Runs the decorated function within a concurrent thread,
     taking care of the result and error management.
 
@@ -34,34 +43,35 @@ def thread(*args, **kwargs) -> Callable:
     The daemon parameter controls the underlying thread daemon flag.
     Default is True.
 
+    The pool parameter accepts a pebble.ThreadPool instance to be used
+    instead of running the function in a new process.
+
     """
-    name = kwargs.get('name')
-    daemon = kwargs.get('daemon', True)
-
-    # decorator without parameters
-    if not kwargs and len(args) == 1 and callable(args[0]):
-        return _thread_wrapper(args[0], name, daemon)
-
-    # decorator with parameters
-    _validate_parameters(name, daemon)
-
-    # without @pie syntax
-    if len(args) == 1 and callable(args[0]):
-        return _thread_wrapper(args[0], name, daemon)
-
-    # with @pie syntax
-    def decorating_function(function: Callable) -> Callable:
-        return _thread_wrapper(function, name, daemon)
-
-    return decorating_function
+    return common.decorate_function(_thread_wrapper, *args, **kwargs)
 
 
-def _thread_wrapper(function: Callable, name: str, daemon: bool) -> Callable:
+def _thread_wrapper(
+        function: Callable,
+        name: str,
+        daemon: bool,
+        _timeout: float,
+        _mp_context,
+        pool: ThreadPool
+) -> Callable:
+    if pool is not None:
+        if not isinstance(pool, ThreadPool):
+            raise TypeError('Pool expected to be ThreadPool')
+
     @wraps(function)
     def wrapper(*args, **kwargs) -> Future:
-        future = Future()
+        if pool is not None:
+            future = pool.schedule(function, args=args, kwargs=kwargs)
+        else:
+            future = Future()
 
-        launch_thread(name, _function_handler, daemon, function, args, kwargs, future)
+            common.launch_thread(
+                name, _function_handler, daemon,
+                function, args, kwargs, future)
 
         return future
 
@@ -77,17 +87,9 @@ def _function_handler(
     """Runs the actual function in separate thread and returns its result."""
     future.set_running_or_notify_cancel()
 
-    try:
-        result = function(*args, **kwargs)
-    except BaseException as error:
-        error.traceback = format_exc()
-        future.set_exception(error)
+    result = common.execute(function, *args, **kwargs)
+
+    if result.status == common.ResultStatus.SUCCESS:
+        future.set_result(result.value)
     else:
-        future.set_result(result)
-
-
-def _validate_parameters(name: str, daemon: bool):
-    if name is not None and not isinstance(name, str):
-        raise TypeError('Name expected to be None or string')
-    if daemon is not None and not isinstance(daemon, bool):
-        raise TypeError('Daemon expected to be None or bool')
+        future.set_exception(result.value)
