@@ -1,5 +1,5 @@
 # This file is part of Pebble.
-# Copyright (c) 2013-2023, Matteo Cafasso
+# Copyright (c) 2013-2024, Matteo Cafasso
 
 # Pebble is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License
@@ -17,25 +17,24 @@
 import os
 import time
 import atexit
+import signal
 import pickle
 import multiprocessing
 
 from itertools import count
-from collections import namedtuple
-from signal import SIG_IGN, SIGINT, signal
+from dataclasses import dataclass
 from typing import Any, Callable, Optional
 from concurrent.futures.process import BrokenProcessPool
 from concurrent.futures import CancelledError, TimeoutError
 
-from pebble.pool.base_pool import ProcessMapFuture, map_results
-from pebble.pool.base_pool import CREATED, ERROR, RUNNING, SLEEP_UNIT
 from pebble.pool.base_pool import Worker, iter_chunks, run_initializer
 from pebble.pool.base_pool import PoolContext, BasePool, Task, TaskPayload
+from pebble.pool.base_pool import PoolStatus, ProcessMapFuture, map_results
 from pebble.pool.channel import ChannelError, WorkerChannel, channels
+from pebble.common import Result, ResultStatus, CONSTS
 from pebble.common import launch_process, stop_process
 from pebble.common import ProcessExpired, ProcessFuture
 from pebble.common import process_execute, launch_thread
-from pebble.common import Result, SUCCESS, FAILURE, ERROR as TASK_ERROR
 
 
 class ProcessPool(BasePool):
@@ -49,23 +48,25 @@ class ProcessPool(BasePool):
     initializer must be callable, if passed, it will be called
     every time a worker is started, receiving initargs as arguments.
 
+    The context parameter can be used to specify the multiprocessing.context object
+    used for starting the worker processes.
+
     """
 
     def __init__(self, max_workers: int = multiprocessing.cpu_count(),
                  max_tasks: int = 0,
                  initializer: Callable = None,
                  initargs: list = (),
-                 context: multiprocessing.context.BaseContext = None):
+                 context: multiprocessing.context.BaseContext = multiprocessing):
         super().__init__(max_workers, max_tasks, initializer, initargs)
-        mp_context = multiprocessing if context is None else context
-        self._pool_manager = PoolManager(self._context, mp_context)
+        self._pool_manager = PoolManager(self._context, context)
         self._task_scheduler_loop = None
         self._pool_manager_loop = None
         self._message_manager_loop = None
 
     def _start_pool(self):
-        with self._context.state_mutex:
-            if self._context.state == CREATED:
+        with self._context.status_mutex:
+            if self._context.status == PoolStatus.CREATED:
                 self._pool_manager.start()
 
                 self._task_scheduler_loop = launch_thread(
@@ -75,7 +76,7 @@ class ProcessPool(BasePool):
                 self._message_manager_loop = launch_thread(
                     None, message_manager_loop, True, self._pool_manager)
 
-                self._context.state = RUNNING
+                self._context.status = PoolStatus.RUNNING
 
     def _stop_pool(self):
         if self._pool_manager_loop is not None:
@@ -101,7 +102,7 @@ class ProcessPool(BasePool):
         A *pebble.ProcessFuture* object is returned.
 
         """
-        self._check_pool_state()
+        self._check_pool_status()
 
         future = ProcessFuture()
         payload = TaskPayload(function, args, kwargs)
@@ -113,7 +114,7 @@ class ProcessPool(BasePool):
 
     def submit(self, function: Callable,
                timeout: Optional[float],
-               *args, **kwargs) -> ProcessFuture:
+               /, *args, **kwargs) -> ProcessFuture:
         """This function is provided for compatibility with
         `asyncio.loop.run_in_executor`.
 
@@ -138,7 +139,7 @@ class ProcessPool(BasePool):
         A *pebble.ProcessFuture* object is returned.
 
         """
-        self._check_pool_state()
+        self._check_pool_status()
 
         timeout = kwargs.get('timeout')
         chunksize = kwargs.get('chunksize', 1)
@@ -148,7 +149,7 @@ class ProcessPool(BasePool):
 
         futures = [self.schedule(
             process_chunk, args=(function, chunk), timeout=timeout)
-            for chunk in iter_chunks(chunksize, *iterables)]
+            for chunk in iter_chunks(zip(*iterables), chunksize)]
 
         return map_results(ProcessMapFuture(futures), timeout)
 
@@ -170,7 +171,7 @@ def task_scheduler_loop(pool_manager: 'PoolManager'):
             else:
                 task_queue.task_done()
     except BrokenProcessPool:
-        context.state = ERROR
+        context.status = PoolStatus.ERROR
 
 
 def pool_manager_loop(pool_manager: 'PoolManager'):
@@ -179,9 +180,9 @@ def pool_manager_loop(pool_manager: 'PoolManager'):
     try:
         while context.alive and not GLOBAL_SHUTDOWN:
             pool_manager.update_status()
-            time.sleep(SLEEP_UNIT)
+            time.sleep(CONSTS.sleep_unit)
     except BrokenProcessPool:
-        context.state = ERROR
+        context.status = PoolStatus.ERROR
 
 
 def message_manager_loop(pool_manager: 'PoolManager'):
@@ -189,9 +190,9 @@ def message_manager_loop(pool_manager: 'PoolManager'):
 
     try:
         while context.alive and not GLOBAL_SHUTDOWN:
-            pool_manager.process_next_message(SLEEP_UNIT)
+            pool_manager.process_next_message(CONSTS.sleep_unit)
     except BrokenProcessPool:
-        context.state = ERROR
+        context.status = PoolStatus.ERROR
 
 
 class PoolManager:
@@ -240,12 +241,13 @@ class PoolManager:
             self.worker_manager.stop_worker(task.worker_id)
             self.task_manager.task_done(
                 task.id,
-                Result(FAILURE, TimeoutError("Task timeout", task.timeout)))
+                Result(ResultStatus.FAILURE,
+                       TimeoutError("Task timeout", task.timeout)))
 
         for task in self.task_manager.cancelled_tasks():
             self.worker_manager.stop_worker(task.worker_id)
             self.task_manager.task_done(
-                task.id, Result(FAILURE, CancelledError()))
+                task.id, Result(ResultStatus.FAILURE, CancelledError()))
 
     def update_workers(self):
         """Handles unexpected processes termination."""
@@ -262,17 +264,17 @@ class PoolManager:
         except LookupError:
             return
         else:
-            error = ProcessExpired('Abnormal termination', code=exitcode)
-            self.task_manager.task_done(task.id, Result(TASK_ERROR, error))
+            error = ProcessExpired('Abnormal termination', code=exitcode, pid=worker_id)
+            self.task_manager.task_done(
+                task.id, Result(ResultStatus.ERROR, error))
 
     def find_expired_task(self, worker_id: int) -> Task:
-        tasks = tuple(self.task_manager.tasks.values())
+        tasks = dictionary_values(self.task_manager.tasks)
         running_tasks = tuple(t for t in tasks if t.worker_id != 0)
-
         if running_tasks:
             return task_worker_lookup(running_tasks, worker_id)
-        else:
-            raise BrokenProcessPool("All workers expired")
+
+        raise BrokenProcessPool("All workers expired")
 
 
 class TaskManager:
@@ -304,7 +306,7 @@ class TaskManager:
         else:
             if task.future.cancelled():
                 task.set_running_or_notify_cancel()
-            elif result.status == SUCCESS:
+            elif result.status == ResultStatus.SUCCESS:
                 task.future.set_result(result.value)
             else:
                 task.future.set_exception(result.value)
@@ -314,13 +316,14 @@ class TaskManager:
     def task_problem(self, task_id: int, error: Exception):
         """Set the task with the error it caused within the Pool."""
         self.task_start(task_id, None)
-        self.task_done(task_id, Result(TASK_ERROR, error))
+        self.task_done(task_id, Result(ResultStatus.ERROR, error))
 
     def timeout_tasks(self) -> tuple:
-        return tuple(t for t in tuple(self.tasks.values()) if self.timeout(t))
+        return tuple(t for t in dictionary_values(self.tasks)
+                     if self.timeout(t))
 
     def cancelled_tasks(self) -> tuple:
-        return tuple(t for t in tuple(self.tasks.values())
+        return tuple(t for t in dictionary_values(self.tasks)
                      if t.timestamp != 0 and t.future.cancelled())
 
     @staticmethod
@@ -371,8 +374,8 @@ class WorkerManager:
         Returns the workers which have unexpectedly ended.
 
         """
-        workers = tuple(self.workers.values())
-        expired = tuple(w for w in workers if not w.is_alive())
+        expired = tuple(w for w in dictionary_values(self.workers)
+                        if not w.is_alive())
 
         for worker in expired:
             self.workers.pop(worker.pid)
@@ -415,7 +418,8 @@ class WorkerManager:
 
 def worker_process(params: Worker, channel: WorkerChannel):
     """The worker process routines."""
-    signal(SIGINT, SIG_IGN)
+    signal.signal(signal.SIGINT, signal.SIG_IGN)
+    signal.signal(signal.SIGTERM, signal.SIG_DFL)
 
     channel.initialize()
 
@@ -495,13 +499,50 @@ def interpreter_shutdown():
         stop_process(worker)
 
 
+def dictionary_values(dictionary: dict) -> tuple:
+    """Returns a snapshot of the dictionary values handling race conditions."""
+    while True:
+        try:
+            return tuple(dictionary.values())
+        except RuntimeError:  # race condition
+            pass
+
+
 atexit.register(interpreter_shutdown)
 
 
 GLOBAL_SHUTDOWN = False
 WORKERS_NAME = 'pebble_pool_worker'
-NoMessage = namedtuple('NoMessage', ())
-TaskResult = namedtuple('TaskResult', ('task', 'result'))
-TaskProblem = namedtuple('TaskProblem', ('task', 'error'))
-WorkerTask = namedtuple('WorkerTask', ('id', 'payload'))
-Acknowledgement = namedtuple('Acknowledgement', ('worker', 'task'))
+
+
+@dataclass
+class NoMessage:
+    pass
+
+
+@dataclass
+class TaskResult:
+    """The result of a Task."""
+    task: id
+    result: Any
+
+
+@dataclass
+class TaskProblem:
+    """Issue occurred within a Task."""
+    task: id
+    error: BaseException
+
+
+@dataclass
+class WorkerTask:
+    """A Task assigned to a worker."""
+    id: id
+    payload: TaskPayload
+
+
+@dataclass
+class Acknowledgement:
+    """Ack from a worker of a received Task."""
+    worker: id
+    task: id
