@@ -1,5 +1,5 @@
 # This file is part of Pebble.
-# Copyright (c) 2013-2023, Matteo Cafasso
+# Copyright (c) 2013-2024, Matteo Cafasso
 
 # Pebble is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License
@@ -22,10 +22,9 @@ from itertools import count
 from typing import Callable
 from concurrent.futures import Future
 
-from pebble.common import execute, launch_thread, SUCCESS
-from pebble.pool.base_pool import MapFuture, map_results
+from pebble.common import ResultStatus, execute, launch_thread, CONSTS
 from pebble.pool.base_pool import iter_chunks, run_initializer
-from pebble.pool.base_pool import CREATED, ERROR, RUNNING, SLEEP_UNIT
+from pebble.pool.base_pool import PoolStatus, MapFuture, map_results
 from pebble.pool.base_pool import PoolContext, BasePool, Task, TaskPayload
 
 
@@ -50,13 +49,13 @@ class ThreadPool(BasePool):
         self._pool_manager_loop = None
 
     def _start_pool(self):
-        with self._context.state_mutex:
-            if self._context.state == CREATED:
+        with self._context.status_mutex:
+            if self._context.status == PoolStatus.CREATED:
                 self._pool_manager.start()
                 self._pool_manager_loop = launch_thread(
                     None, pool_manager_loop, True, self._pool_manager)
 
-                self._context.state = RUNNING
+                self._context.status = PoolStatus.RUNNING
 
     def _stop_pool(self):
         if self._pool_manager_loop is not None:
@@ -72,7 +71,7 @@ class ThreadPool(BasePool):
         A *concurrent.futures.Future* object is returned.
 
         """
-        self._check_pool_state()
+        self._check_pool_status()
 
         future = Future()
         payload = TaskPayload(function, args, kwargs)
@@ -99,7 +98,7 @@ class ThreadPool(BasePool):
         the size will be controlled by the Pool.
 
         """
-        self._check_pool_state()
+        self._check_pool_status()
 
         timeout = kwargs.get('timeout')
         chunksize = kwargs.get('chunksize', 1)
@@ -108,7 +107,7 @@ class ThreadPool(BasePool):
             raise ValueError("chunksize must be >= 1")
 
         futures = [self.schedule(process_chunk, args=(function, chunk))
-                   for chunk in iter_chunks(chunksize, *iterables)]
+                   for chunk in iter_chunks(zip(*iterables), chunksize)]
 
         return map_results(MapFuture(futures), timeout)
 
@@ -118,7 +117,7 @@ def pool_manager_loop(pool_manager: 'PoolManager'):
 
     while context.alive:
         pool_manager.update_status()
-        time.sleep(SLEEP_UNIT)
+        time.sleep(CONSTS.sleep_unit)
 
 
 class PoolManager:
@@ -164,7 +163,7 @@ def worker_thread(context: PoolContext):
 
     if parameters.initializer is not None:
         if not run_initializer(parameters.initializer, parameters.initargs):
-            context.state = ERROR
+            context.status = PoolStatus.ERROR
             return
 
     for task in get_next_task(context, parameters.max_tasks):
@@ -194,7 +193,7 @@ def execute_next_task(task: Task):
 
     result = execute(payload.function, *payload.args, **payload.kwargs)
 
-    if result.status == SUCCESS:
+    if result.status == ResultStatus.SUCCESS:
         task.future.set_result(result.value)
     else:
         task.future.set_exception(result.value)
